/* ==========================================================================*/
/* - - - - - - - - - - -  A m b e 2 0 0 0 . C   - - - - - - - - - - - - - -  */
/* ==========================================================================*/
/*	Created   2007-07-01   M. Wheatley                                       */
/*	Modified  2007-09-09   MW                                                */
/*.........................................................................  */
/*  AMBE 2000 vocoder handler                                                */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "irq.h"
#include "ambe2000.h"
#include "usart.h"
#include <string.h>

 
/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define AMBECODEC_TASK_PRIORITY ( tskIDLE_PRIORITY + 1 )
#define AMBECHAN_TASK_PRIORITY ( tskIDLE_PRIORITY + 2 )
#define QUEUE_SIZE 3


/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
BOOL QueueOverflow;
BOOL CodecDataStarted;
BOOL ChanDataStarted;
volatile U32BIT XferState;
U32BIT SSCTransferIndex;
volatile U32BIT ChanTxHead;
volatile U32BIT ChanTxTail;
tInFrame InFrame[CHANNEL_QUE_SIZE];
tOutFrame OutFrame;
xSemaphoreHandle xAmbeCodecDataRdySemaphore;
xSemaphoreHandle xAmbeChanDataRdySemaphore;
volatile tCodecData* pActiveCodecBuf;
volatile U32BIT CodecTxHead;
volatile U32BIT CodecTxTail;
tCodecData CodecRxBufA;		//ping pong buffers for data from ambe codec
tCodecData CodecRxBufB;
tCodecData CodecTxQue[CODEC_QUE_SIZE];	//full Queue fro data going to ambe codec

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
static void AmbeCodecTask( void *pvParmeters );
static void AmbeChanTask( void *pvParmeters );

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -------------------------                                */
/*                 | A m b e 2 0 0 0 I n i t |                               */
/*                  -------------------------                                */
/*    Called to initialize the SSI port and AMBE20x0.                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Ambe2000Init(void )
{
int i,j;
	vSemaphoreCreateBinary( xAmbeCodecDataRdySemaphore );
	vSemaphoreCreateBinary( xAmbeChanDataRdySemaphore );
	//* Configure PIO controllers to periph mode for SSC
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
  		((unsigned int) SSC_TF   ) |
 		((unsigned int) SSC_TK   ) |
 		((unsigned int) SSC_TD   ) |
 		((unsigned int) SSC_RD   ),// Peripheral A
 		0); // Peripheral B
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, SLIP_EN ) ;
	AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, SLIP_EN ) ;
	
	QueueOverflow = FALSE;
	for(i=0; i<CODEC_DATA_LENGTH ;i++)
	{
		CodecRxBufA.Data[i] = 0x0000;
		CodecRxBufB.Data[i] = 0x0000;
	}
	CodecRxBufA.Hdr = CODEC_DATA_LENGTH*2 + 2 + (TYPE_TARG_DATA_ITEM0<<8);	//preload data msg header
	CodecRxBufB.Hdr = CODEC_DATA_LENGTH*2 + 2 + (TYPE_TARG_DATA_ITEM0<<8);	//preload data msg header

	for(i=0; i<CODEC_QUE_SIZE ;i++)
		for(j=0; j<CODEC_DATA_LENGTH ;j++)
			CodecTxQue[i].Data[j] = 0x0000;

	//Setup AT91SAM7X Sync Serial DMA Controller
	*AT91C_SSC_RPR = (U32BIT)(&CodecRxBufA.Data);
	*AT91C_SSC_RCR = CODEC_DATA_LENGTH;
	*AT91C_SSC_RNPR = (U32BIT)(&CodecRxBufB.Data);
	*AT91C_SSC_RNCR = CODEC_DATA_LENGTH;
	
	*AT91C_SSC_TPR = (U32BIT)(&CodecTxQue[0]);
	*AT91C_SSC_TCR = CODEC_DATA_LENGTH;
	*AT91C_SSC_TNPR = (U32BIT)(&CodecTxQue[1]);
	*AT91C_SSC_TNCR = CODEC_DATA_LENGTH;
	CodecTxHead = 0;
	CodecTxTail = 1;
	pActiveCodecBuf = &CodecRxBufB;
	*AT91C_SSC_PTCR = AT91C_PDC_RXTEN | AT91C_PDC_TXTEN;

	//Setup AT91SAM7X Sync Serial Controller
	AT91F_PMC_EnablePeriphClock( AT91C_BASE_PMC, (1<<AT91C_ID_SSC) );	//enable clk
	*AT91C_SSC_CR = AT91C_SSC_SWRST;		//hold in reset while configuring
	*AT91C_SSC_CMR = 24;	//1.024 MHz clk
	*AT91C_SSC_RCMR = (1<<16)|AT91C_SSC_CKS_TK|AT91C_SSC_START_TX;
	*AT91C_SSC_RFMR = (AT91C_SSC_DATLEN & (16-1)) | AT91C_SSC_MSBF;
	
	*AT91C_SSC_TCMR = ((63<<24)&AT91C_SSC_PERIOD)|((1<<16)&AT91C_SSC_STTDLY )|AT91C_SSC_CKS_DIV|
									AT91C_SSC_CKO_CONTINOUS|AT91C_SSC_CKI|AT91C_SSC_START_RISE_RF;
	*AT91C_SSC_TFMR = (AT91C_SSC_DATLEN & (16-1)) | AT91C_SSC_MSBF | AT91C_SSC_FSOS_POSITIVE;
	*AT91C_SSC_CR = AT91C_SSC_RXEN|AT91C_SSC_TXEN;
    //* Config SSC interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_SSC, SSC_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, SSCIrqService);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_SSC);	//enable SSC DMA irq service

	*AT91C_SSC_IER = AT91C_SSC_ENDRX;	//irq enable just Rx since both occur at same time

	//* enable the clock of the SPI port
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_SPI ) ;
 	//* Configure PIO controllers to periph mode for SPI for AMBE CHAN com
 	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, CHAN_STB ) ;
	AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, CHAN_STB ) ;
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 		((unsigned int) CHAN_TXDATA  ) |
 		((unsigned int) CHAN_RXDATA   )|	// Peripheral A
 		((unsigned int) CHAN_CLK   ), 0
 	); 
	*AT91C_SPI_CR = AT91C_SPI_SWRST;
	*AT91C_SPI_CR = AT91C_SPI_SPIEN; // (SPI) Control Register
	*AT91C_SPI_MR = AT91C_SPI_MSTR|AT91C_SPI_MODFDIS; // (SPI) Mode Register
	AT91C_SPI_CSR[0] = AT91C_SPI_BITS_16|(48<<8); 	//SPI clk == MCK/48
    //* Config SPI interrupt
    *AT91C_SPI_IER = AT91C_SPI_RDRF;
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_SPI, SPI_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, SPIIrqService);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_SPI);	//enable SPI irq service

	InitInFrameQueue();
	ChanTxHead = 0;
	ChanTxTail = 0;

	xTaskCreate( AmbeCodecTask, ( const signed portCHAR * const ) "CODEC", 
				configMINIMAL_STACK_SIZE, NULL, AMBECODEC_TASK_PRIORITY, ( xTaskHandle * ) NULL );
	xTaskCreate( AmbeChanTask, ( const signed portCHAR * const ) "CHAN", 
				configMINIMAL_STACK_SIZE, NULL, AMBECHAN_TASK_PRIORITY, ( xTaskHandle * ) NULL );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  ---------------------------------                        */
/*                 | I n i t I n F r a m e Q u e u e |                       */
/*                  ---------------------------------                        */
/*    Initializes the InFrameQue that goes to the AMBE channel               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void InitInFrameQueue(void)
{
int i;
	for(i=0; i<CHANNEL_QUE_SIZE; i++)
	{
		memset(InFrame[i].All, 0, sizeof(InFrame[i].All) );
		InFrame[i].fld.Hdr = AMBE2020_FRAME_SYNC;
		InFrame[i].fld.CW1 = 0x0000;

		InFrame[i].fld.Rate0 = 0x1030;	//FEC block codes
		InFrame[i].fld.Rate1= 0x4000;
		InFrame[i].fld.Rate2 = 0x0000;
		InFrame[i].fld.Rate3 = 0x0000;
		InFrame[i].fld.Rate4 = 0x0048;
		InFrame[i].fld.DTMFCtrl = 0x00FF;
		InFrame[i].fld.CW2 = 0x0000;		//volume off
	}
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------                          */
/*                     | A m b e C o d e c T a s k |                         */
/*                      ---------------------------                          */
/*    Task that manages the AMBE Codec data blocks                           */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void AmbeCodecTask( void *pvParmeters )
{
	CodecDataStarted = FALSE;
	ChanDataStarted = FALSE;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, AMBE_RESET_ );	//release AMBE 20xx from reset
	while( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & EPR_) != 0 )
	{	//wait for chip to be ready
		vTaskDelay( 2 );
	}
	xSemaphoreTake(xAmbeCodecDataRdySemaphore, ( portTickType )0);
	for( ;; )	//loop forever in this task
    {
		if( xSemaphoreTake( xAmbeCodecDataRdySemaphore, ( portTickType )500 ) == pdTRUE )
		{	//codec block ready from/to ambe
			if(	GENERAL_STATUS_RUNNING == Status )
			{	//send codec data from AMBE to usart
				SendUsartMsg( (U8BIT*)pActiveCodecBuf, CODEC_BUF_SIZE*2 );
			}
		}
 	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------------                            */
/*                     | A m b e C h a n T a s k |                           */
/*                      -------------------------                            */
/*    Task that manages the AMBE Channel data                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void AmbeChanTask( void *pvParmeters )
{
U16BIT Hdr;
U32BIT i;
t16BITS tmp;
U8BIT val;
	Hdr = sizeof(OutFrame.All) + 2 + (TYPE_TARG_DATA_ITEM1<<8);	//preload data msg header
	while( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & EPR_) != 0 )
	{	//wait for chip to ask for first data packet
		vTaskDelay( 2 );
	}
	XferState = XFERSTATE_LOOKING_FOR_SYNC;
	SSCTransferIndex = 0;
	AT91C_BASE_TC2->TC_RC = TIMER2_SYNCSEARCH;
	AT91C_BASE_TC2->TC_CCR = AT91C_TC_SWTRG;	//trigger one shot timer 

	xSemaphoreTake(xAmbeChanDataRdySemaphore, ( portTickType )0);
	for( ;; )	//loop forever in this task
    {
		if( xSemaphoreTake( xAmbeChanDataRdySemaphore, ( portTickType )500 ) == pdTRUE )
		{	//channel data frame ready from/to ambe
			if(	GENERAL_STATUS_RUNNING == Status )
			{
				for(i=0; i<12;i++)	//change byte data fields to little endian
				{
					tmp.all = OutFrame.fld.ChData[i];
					val = tmp.bytes.b0;
					tmp.bytes.b0 = tmp.bytes.b1;
					tmp.bytes.b1 = val;
					OutFrame.fld.ChData[i] = tmp.all;
				}
				SendUsartMsg( (U8BIT*)&Hdr, 2);
				SendUsartMsg( (U8BIT*)OutFrame.All, sizeof(OutFrame.All));
			}
		}
 	}
}

