/*------------------------------------------------------------------------------
//*-                   C S t a r t u p . S
//*------------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*-----------------------------------------------------------------------------
//*- File source          : Cstartup.S
//*- Object               : CStartup for GCC AT91SAM7256 application code (Uses REMAP)
//*-
//*- 1.0 2005/01/29 MSW   : Creation
//*-----------------------------------------------------------------------------*/
	/* Initial Stack Sizes */
	.set  UND_STACK_SIZE, 0x00000004
	.set  ABT_STACK_SIZE, 0x00000004
	.set  FIQ_STACK_SIZE, 0x00000100
	.set  IRQ_STACK_SIZE, 0X00000100
	.set  SVC_STACK_SIZE, 0x00000100

	.set AIC_IVR,         (256) 
	.set AIC_FVR,         (260) 
	.set AIC_EOICR,       (304)
	.set AT91C_BASE_AIC,  (0xFFFFF000)


#;------------------------------------------------------------------------------
#;- Section Definition
#;-----------------
#;-  Section    
#;- .internal_ram_top 	Top_Stack: used by the cstartup for vector initalisation 
#;-			management defined by ld and affect from ldscript 
#;------------------------------------------------------------------------------
	.section 	.internal_ram_top
	.code 32
	.align 	0
	.global	Top_Stack
Top_Stack:
	
/*------------------------------------------------------------------------------
*- Area Definition
*------------------------------------------------------------------------------
*  */
        .section .startup
 		.global _chkloc
        .global _startup
		.extern swi_handler
		.extern Toggle
        .func   _startup
_startup:
reset: 
/*------------------------------------------------------------------------------
//*- Exception vectors 
//*------------------------------------------------------------------------------*/
//* This block (0x0010 1000 to 0x0010 103C) gets copied into Remaped RAM 0x00 to 0x3C
//*------------------------------------------------------------------------------*/
		ldr pc,	_rst          	/* 0x00 Reset handler */
undefvec:
		ldr pc,	_undf         	/* 0x04 Undefined Instruction */
swivec:
		ldr pc,	_swi          	/* 0x08 Software Interrupt */
pabtvec:
		ldr pc,	_pabt         	/* 0x0C Prefetch Abort */
dabtvec:
		ldr pc,	_dabt         	/* 0x10 Data Abort */
rsvdvec:
		b	rsvdvec          	/* 0x14 reserved */
irqvec:
		ldr pc,	[pc,#-0xF20]   	/* 0x18 IRQ	(jmp directly to service routine) */
fiqvec:               			            	
		ldr pc,	[pc,#-0xF20]   	/* 0x1C FIQ	(jmp directly to service routine) */
/*		ldr pc,	_fiq          	0x1c FIQ */

/*table of execution jump addresses */
_rst:	.word InitReset				/* 0x20  reset handler      	*/
_undf:  .word _stub					/* 0x24 undefined				*/
_swi:   .word swi_handler			/* 0x28 SWI						*/
_pabt:  .word _stub                 /* 0x2C program abort			*/
_dabt:  .word _stub                 /* 0x30 data abort				*/
_irq:   .word _stub                 /* 0x34 IRQ						*/
_fiq:   .word FIQ_Handler_Entry     /* 0x38 FIQ						*/
_chkloc:   .word 0x55AA5AA5         /* 0x3C location of chksum ptr  */

//*------------------------------------------------------------------------------*/
/*  End of block copied to RAM by bootloader   
//*------------------------------------------------------------------------------*/

_stub: 	b	.						/* irq stub routine			*/

//*------------------------------------------------------------------------------*/
FIQ_Handler_Entry:
/*- Switch in SVC/User Mode to allow User Stack access for C code 	*/
/* because the FIQ is not yet acknowledged */

/*- Save and r0 in FIQ_Register */
            mov         r9,r0
	        ldr         r0 , [r8, #AIC_FVR]
            msr         CPSR_c,#I_BIT | F_BIT | ARM_MODE_SVC

/*- Save scratch/used registers and LR in User Stack */
            stmfd       sp!, { r1-r3, r12, lr}

/*- Branch to the routine pointed by the AIC_FVR */
            mov         r14, pc
            bx          r0

/*- Restore scratch/used registers and LR from User Stack */
            ldmia       sp!, { r1-r3, r12, lr}

/*- Leave Interrupts disabled and switch back in FIQ mode */
            msr         CPSR_c, #I_BIT | F_BIT | ARM_MODE_FIQ

/*- Restore the R0 ARM_MODE_SVC register */
            mov         r0,r9

/*- Restore the Program Counter using the LR_fiq directly in the PC */
            subs        pc,lr,#4

InitReset:
/*------------------------------------------------------------------------------
/*- Low level Init (PMC, AIC, ....) by C function AT91F_LowLevelInit
/*------------------------------------------------------------------------------*/
	.extern   AT91F_LowLevelInit
/*- minumum C initialization */
/*- call  AT91F_LowLevelInit( void) */

	ldr     r13,.RAM_TOP 	/* temporary stack in internal RAM */
/*--Call Low level init function in ABSOLUTE through the Interworking	*/

/*  Dont really need to call this since it has already been done by boot code */
/*	ldr	    r0,=AT91F_LowLevelInit */
/*	mov     lr, pc	*/
/*	bx	    r0		*/

//*------------------------------------------------------------------------------*/
//*- Interrupt and Supervisor Stack are located at the top of internal memory in 
//*------------------------------------------------------------------------------*/
	.set	IRQ_STACK_SIZE, (30*8*4)
	.set	ARM_MODE_USR, 0x10	/* User Mode */
	.set	ARM_MODE_FIQ, 0x11	/* FIQ Mode */
	.set	ARM_MODE_IRQ, 0x12	/* IRQ Mode */
	.set	ARM_MODE_SVC, 0x13	/* Supervisor Mode */
	.set	ARM_MODE_ABT, 0x17	/* Abort Mode */
	.set	ARM_MODE_UND, 0x1B	/* Undefined Mode */
	.set	ARM_MODE_SYS, 0x1F	/* System Mode */
	.set	I_BIT, 0x80
	.set	F_BIT, 0x40

/*------------------------------------------------------------------------------
//*- Setup the stack for each mode
//*-------------------------------*/
	mov   r0,sp
	msr   CPSR_c, #ARM_MODE_UND|I_BIT|F_BIT 	/* Undefined Instruction Mode  */
    mov   sp, r0
    sub   r0, r0, #UND_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_ABT|I_BIT|F_BIT 	/* Abort Mode */
    mov   sp, r0
    sub   r0, r0, #ABT_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_FIQ|I_BIT|F_BIT 	/* FIQ Mode */
   	ldr   r8, =AT91C_BASE_AIC
    mov   sp, r0	
   	sub   r0, r0, #FIQ_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_IRQ|I_BIT|F_BIT 	/* IRQ Mode */
    mov   sp, r0
    sub   r0, r0, #IRQ_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_SVC|I_BIT|F_BIT 	/* Supervisor Mode */
    mov   sp, r0
    sub   r0, r0, #SVC_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_SYS|I_BIT|F_BIT
    mov   sp, r0

	/* We want to start in supervisor mode.  Operation will switch to system
	mode when the first task starts. */
	msr   CPSR_c, #ARM_MODE_SVC|I_BIT|F_BIT


# Relocate .data section (Copy from ROM to RAM)
                LDR     R1, =_etext
                LDR     R2, =_data
                LDR     R3, =_edata
LoopRel:        CMP     R2, R3
                LDRLO   R0, [R1], #4
                STRLO   R0, [R2], #4
                BLO     LoopRel

# Clear .bss section (Zero init)
                MOV     R0, #0
                LDR     R1, =__bss_start__
                LDR     R2, =__bss_end__
LoopZI:         CMP     R1, R2
                STRLO   R0, [R1], #4
                BLO     LoopZI

				ldr	lr,=exit		/*Call users main() function*/
				ldr	r0,=main
				bx	r0

        .size   _startup, . - _startup
        .endfunc
		
	.align 0
.RAM_TOP:
	.word	Top_Stack
	
/* "exit" dummy added by mthomas to avoid sbrk write read etc. needed
   by the newlib default "exit" */
        .global exit
        .func   exit
exit:
        b    .
	.size   exit, . - exit
        .endfunc

        .end
