/* ==========================================================================*/
/* - - - - - - - - - - - - - -  I r q . c  - - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-10   MW                                                */
/*.........................................................................  */
/* Digital Voice Dongle IRQ service Routines                                 */
/*     (must be compiled as Non-Thum ARM mode)                               */
/*.........................................................................  */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "common.h"
#include "board.h"
#include "usart.h"
#include "timer.h"
#include "ambe2000.h"
#include "irq.h"

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"


/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define LED_IDLE_BLINK_RATE 20

/*---------------------------------------------------------------------------*/
/*---------------------> L O C A L   M E M O R Y  <--------------------------*/
/*---------------------------------------------------------------------------*/
volatile S32BIT RxMsgLength;
volatile U32BIT LedTimer;
volatile U32BIT LedTimerTog;

static volatile U32BIT Timekiller;

/*---------------------------------------------------------------------------*/
/*------------------------> M A C R O S <------------------------------------*/
/*---------------------------------------------------------------------------*/

// Dont mess too much with this macro as it is critical to being able to
//  create a 17th clk pulse for the transfer and meet timing spec
#define SPI_SEND( val )						                                \
		/* Sends a 16 bit word to the AMBE channel */                       \
	 	{																	\
 	/*manually clock the strobe state before starting the 16 bit SPI xfer*/ \
			AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, CHAN_CLK );				\
			AT91F_PIO_SetOutput( AT91C_BASE_PIOA, CHAN_CLK|CHAN_STB );		\
			Timekiller *= 3;												\
			AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,CHAN_CLK,0);				\
			*AT91C_SPI_TDR = val;											\
			Timekiller *= 3;												\
			AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, CHAN_STB|CHAN_CLK );	\
	 	}       
	 	
	 	
/******************************************************************************
 *
 * MACRO Name: ISR_ENTRY()
 *
 * Description:
 *    This MACRO is used upon entry to an ISR.  The current version of
 *    the gcc compiler for ARM does not produce correct code for
 *    interrupt routines to operate properly with THUMB code.  The MACRO
 *    performs the following steps:
 *
 *    1 - Adjust address at which execution should resume after servicing
 *        ISR to compensate for IRQ entry
 *    2 - Save the non-banked registers r0-r12 and lr onto the IRQ stack.
 *    3 - Get the status of the interrupted program is in SPSR.
 *    4 - Push it onto the IRQ stack as well.
 *
 *****************************************************************************/
#define ISR_ENTRY() asm volatile(" sub   lr, lr,#4\n" \
                                 " stmfd sp!,{r0-r12,lr}\n" \
                                 " mrs   r1, spsr\n" \
                                 " stmfd sp!,{r1}")

/******************************************************************************
 *
 * MACRO Name: ISR_EXIT()
 *
 * Description:
 *    This MACRO is used to exit an ISR.  The current version of the gcc
 *    compiler for ARM does not produce correct code for interrupt
 *    routines to operate properly with THUMB code.  The MACRO performs
 *    the following steps:
 *
 *    1 - Recover SPSR value from stack       
 *    2 - and restore  its value                   
 *    3 - Pop the return address & the saved general registers from
 *        the IRQ stack & return
 *
 *****************************************************************************/
#define ISR_EXIT()  asm volatile(" ldmfd sp!,{r1}\n" \
                                 " msr   spsr_c,r1\n" \
                                 " ldmfd sp!,{r0-r12,pc}^") 
  

/******************************************************************************
 *
 * MACRO Name: FSR_ENTRY()
 *
 * Description:
 *    This MACRO is used upon entry to an FSR.  The current version of
 *    the gcc compiler for ARM does not produce correct code for
 *    interrupt routines to operate properly with THUMB code.  The MACRO
 *    performs the following steps:
 *
 *    1 - Adjust address at which execution should resume after servicing
 *        FSR to compensate for FIQ entry
 *    2 - Save the non-banked registers r0-r7 and lr onto the FIQ stack.
 *
 *****************************************************************************/
#define FSR_ENTRY() asm volatile(" sub   lr, lr,#4\n" \
                                 " stmfd sp!,{r0-r7,lr}")

/******************************************************************************
 *
 * MACRO Name: FSR_EXIT()
 *
 * Description:
 *    This MACRO is used to exit an FSR.  The current version of the gcc
 *    compiler for ARM does not produce correct code for interrupt
 *    routines to operate properly with THUMB code.  The MACRO performs
 *    the following steps:
 *
 *    1 - Pop the return address & the saved general registers from
 *        the FIQ stack & return
 *
 *****************************************************************************/
#define FSR_EXIT()  asm volatile(" ldmfd sp!,{r0-r7,pc}^")


/*---------------------------------------------------------------------------*/
/*----------> I N T E R R U P T    S E R V I C E    R O U T I N E S <--------*/
/*---------------------------------------------------------------------------*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------------                        */
/*                     | U a r t I r q S e r v i c e |                       */
/*                      -----------------------------                        */
/*    Called by uart IRQ.                                                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void UsartIrqService(void)
{
	/* This ISR can cause a context switch, so the first statement must be a
	call to the portENTER_SWITCHING_ISR() macro.  This must be BEFORE any
	variable declarations. */
	portENTER_SWITCHING_ISR();
U32BIT Status;
portBASE_TYPE TaskWoken = pdFALSE;
	Status = *AT91C_US0_CSR;
	if( Status & AT91C_US_ENDRX )	//usart receiver DMA finished
	{
		if(	RxMsgLength < 0 )	//if just received msg header then get length
		{
			RxMsgLength = (S32BIT)pActiveMsg[0];
			RxMsgLength += (S32BIT)pActiveMsg[1]<<8;
			RxMsgLength &= LENGTH_MASK;
			*AT91C_US0_RPR = (U32BIT)pActiveMsg + 2;
			*AT91C_US0_RCR = RxMsgLength-2;	//setup DMA to get remaining msg bytes
			if( (*AT91C_US0_RCR >= (MAXMSG_SIZE ) ) || (*AT91C_US0_RCR < 2 ) )
			{	//bad msg length so reset to try to get next header
				*AT91C_US0_RPR = (U32BIT)pActiveMsg;
				*AT91C_US0_RCR = 2;
				*AT91C_US0_RNPR = 0;
				*AT91C_US0_RNCR = 0;
				RxMsgLength = -1;	//flag msg length as not yet received
			}
		}
		else	//here if finished getting entire msg
		{
			if(pActiveMsg == UsartRxMsgA)	//toggle ping pong buffer
				pActiveMsg = UsartRxMsgB;
			else
				pActiveMsg = UsartRxMsgA;
			*AT91C_US0_RPR = (U32BIT)pActiveMsg;
			*AT91C_US0_RCR = 2;
			RxMsgLength = -1;	//flag msg length as not yet received
			TaskWoken = xSemaphoreGiveFromISR( xUartSemaphore, ( portBASE_TYPE )pdFALSE );
		}
	}
	if( Status & AT91C_US_TIMEOUT )	//usart receiver DMA timeout
	{
		AT91C_BASE_US0->US_CR = AT91C_US_STTTO;
//		pActiveMsg = UsartRxMsgA;
//		*AT91C_US0_RPR = (U32BIT)pActiveMsg;
//		*AT91C_US0_RCR = 2;
//		*AT91C_US0_RNPR = 0;
//		*AT91C_US0_RNCR = 0;
//		RxMsgLength = -1;	//flag msg length as not yet received
	}
	AT91C_BASE_AIC->AIC_EOICR = 0;
	AT91C_BASE_US0->US_CR = AT91C_US_RSTSTA;
	/* Exit the ISR and force a context switch if needed*/
	
	portEXIT_SWITCHING_ISR( TaskWoken );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------                          */
/*                     | S S C I r q S e r v i c e |                         */
/*                      ---------------------------                          */
/*    Called by IRQ when new data received from Sync serial interface.       */
/* This gets/sends data to the AMBE Codec interface                          */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SSCIrqService(void)
{
	/* This ISR can cause a context switch, so the first statement must be a
	call to the portENTER_SWITCHING_ISR() macro.  This must be BEFORE any
	variable declarations. */
	portENTER_SWITCHING_ISR();
int i,j;
portBASE_TYPE TaskWoken = pdFALSE;
	AT91C_BASE_AIC->AIC_EOICR = 0;

	if(	pActiveCodecBuf == &CodecRxBufA  )
	{	//if ping pong buf B finished
		*AT91C_SSC_RNPR = (U32BIT)(&CodecRxBufA.Data);
		pActiveCodecBuf = &CodecRxBufB;
	}
	else
	{	//if ping pong buf A finished
		*AT91C_SSC_RNPR = (U32BIT)(&CodecRxBufB.Data);
		pActiveCodecBuf = &CodecRxBufA;
	}
	*AT91C_SSC_RNCR = CODEC_DATA_LENGTH;

	*AT91C_SSC_TNPR = (U32BIT)(&CodecTxQue[CodecTxTail++].Data );
	*AT91C_SSC_TNCR = CODEC_DATA_LENGTH;
	CodecTxTail &= (CODEC_QUE_SIZE-1);
	if( CodecTxTail==CodecTxHead)	//Codec queue under run
	{	//move pointer back
		CodecTxTail -= (CODEC_QUE_SIZE/4);
		CodecTxTail &= (CODEC_QUE_SIZE-1);
		QueueOverflow = TRUE;
		for(i=0; i<CODEC_QUE_SIZE ;i++)
			for(j=0; j<CODEC_DATA_LENGTH ;j++)
				CodecTxQue[i].Data[j] = 0x0000;
	}
	TaskWoken = xSemaphoreGiveFromISR( xAmbeCodecDataRdySemaphore, ( portBASE_TYPE )pdFALSE );
	/* Exit the ISR and force a context switch */
	portEXIT_SWITCHING_ISR( TaskWoken );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------                          */
/*                     | S P I I r q S e r v i c e |                         */
/*                      ---------------------------                          */
/*    Called by IRQ when new data received/transmitted from SPI interface.   */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SPIIrqService(void)
{
	ISR_ENTRY();
U32BIT tmp;
	AT91C_BASE_AIC->AIC_EOICR = 0;
	if(	XFERSTATE_LOOKING_FOR_SYNC == XferState )
	{
		if( AMBE2020_FRAME_SYNC == (*AT91C_SPI_RDR & 0x0000FFFF) )
		{	//received valid sync for start of new channel data frame
			SSCTransferIndex = 0;
			XferState = XFERSTATE_RECEIVING;
			OutFrame.All[0] = AMBE2020_FRAME_SYNC;
// InFrame[ChanTxTail].fld.DTMFCtrl = 0xF60A;		//create test tone	
			SPI_SEND( InFrame[ChanTxTail].All[SSCTransferIndex++]);
		}
		else
		{	//trigger next sampling for sync word at 1mS intervals using one-shot timer2
			AT91C_BASE_TC2->TC_RC = TIMER2_SYNCSEARCH;
			AT91C_BASE_TC2->TC_CCR = AT91C_TC_SWTRG;	//trigger one shot timer 
		}
	}
	else if( XFERSTATE_RECEIVING == XferState )
	{	//here getting remaining words of channel data frame
		if(SSCTransferIndex < 24)
		{	//send and receive next word in channel data frames
			OutFrame.All[SSCTransferIndex] = (*AT91C_SPI_RDR & 0x0000FFFF);
			SPI_SEND( InFrame[ChanTxTail].All[SSCTransferIndex++]);
		}
		else
		{	//have finished the frame
			tmp = *AT91C_SPI_RDR;	//dummy read to clear irq flag
			SSCTransferIndex = 0;
			XferState = XFERSTATE_FINISHED;
			AT91C_BASE_TC2->TC_RC = TIMER2_DATARDY;
			AT91C_BASE_TC2->TC_CCR = AT91C_TC_SWTRG;	//trigger one shot timer for signaling
			ChanTxTail++;
			ChanTxTail &= (CHANNEL_QUE_SIZE-1);
			if( ChanTxTail==ChanTxHead)	//queue Under run
			{	//move pointer back
				ChanTxTail -= (CHANNEL_QUE_SIZE/4);
				ChanTxTail &= (CHANNEL_QUE_SIZE-1);
				QueueOverflow = TRUE;
				for(tmp=0; tmp<CHANNEL_QUE_SIZE; tmp++)
					InFrame[tmp].fld.CW2 = 0x0000;		//volume off	
			}		
		}
	}	
	ISR_EXIT();
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------------                    */
/*                     | T i m e r 2 I r q S e r v i c e |                   */
/*                      ---------------------------------                    */
/*    Called by IRQ to service one-shot timer 2.                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Timer2IrqService(void)
{
	/* This ISR can cause a context switch, so the first statement must be a
	call to the portENTER_SWITCHING_ISR() macro.  This must be BEFORE any
	variable declarations. */
	portENTER_SWITCHING_ISR();
portBASE_TYPE TaskWoken = pdFALSE;
S32BIT tmp;
    //* Acknowledge interrupt status
    tmp = AT91C_BASE_TC2->TC_SR;
	AT91C_BASE_AIC->AIC_EOICR = 0;
	if(	XFERSTATE_FINISHED == XferState )
	{	//here if just finished getting/sending packet of ambe2020 data
		XferState = XFERSTATE_LOOKING_FOR_SYNC;
		AT91C_BASE_TC2->TC_RC = TIMER2_DELAY_TO_SYNC_SEARCH;
		AT91C_BASE_TC2->TC_CCR = AT91C_TC_SWTRG;	//trigger one shot timer just before
													//ambe2020 is ready
		TaskWoken = xSemaphoreGiveFromISR( xAmbeChanDataRdySemaphore, ( portBASE_TYPE )pdFALSE );
	}
	else
	{	//here just before ambe is ready with new frame sync so start sync watching process
		SSCTransferIndex = 0;
		SPI_SEND( AMBE2020_IDLE_WORD );
	}
	portEXIT_SWITCHING_ISR( TaskWoken );
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------------                    */
/*                     | T i m e r 1 I r q S e r v i c e |                   */
/*                      ---------------------------------                    */
/*    GP timer.                                                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Timer1IrqService(void)
{
	ISR_ENTRY();
U32BIT dummy;
AT91PS_TC TC_pt = AT91C_BASE_TC1;
    //* Acknowledge interrupt status
    dummy = TC_pt->TC_SR;
	AT91C_BASE_AIC->AIC_EOICR = 0;
	
	if(GENERAL_STATUS_RUNNING == Status)		//just toggle LED when running
	{
		if(LedTimerTog)
		{
			SetLedPWM(LED_RUN_INTENSITY);
			LedTimerTog = 0;
		}
		else
		{
			SetLedPWM(LED_OFF);
			LedTimerTog = 1;
		}
	}
	else	//idle blink uses linear up/down ramp for LED pwm
	{
		if(LedTimerTog)
		{
			SetLedPWM(LedTimer);
			if(LedTimer >= LED_MAX)
				LedTimerTog = 0;
			else
				LedTimer += LED_IDLE_BLINK_RATE;
		}
		else
		{
			SetLedPWM(LedTimer);
			if(LedTimer <= LED_MIN)
				LedTimerTog = 1;
			else
				LedTimer -= LED_IDLE_BLINK_RATE;
		}
	}

	ISR_EXIT();
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -------------------------------------                   */
/*                  | S p u r i o u s I r q S e r v i c e |                  */
/*                   -------------------------------------                   */
/*  Must have do nothing handler for spurious interrupts.                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SpuriousIrqService(void)
{

}

