/* ==========================================================================*/
/* - - - - - - - - - - - - - -  M a i n . C   - - - - - - - - - - - - - - - -*/
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-09-09   MW                                                */
/*.........................................................................  */
/* Digital Voice Dongler AT91SAM7256 Firmware                                */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "common.h"
#include "usart.h"
#include "timer.h"
#include "ambe2000.h"
#include "irq.h"
#include <stdlib.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

//Comment out for normal operation
//#define TESTMODE

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define LED_TASK_PRIORITY ( tskIDLE_PRIORITY + 4 )

#define QUEUE_SIZE 3

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
static U8BIT Val;

/*---------------------------------------------------------------------------*/
/*------------------> L O C A L    P R O T O T Y P E S <---------------------*/
/*---------------------------------------------------------------------------*/
void vApplicationIdleHook( void );
void LedTask( void *pvParmeters );
void Led1TestTask( void *pvParmeters );
void Led2TestTask( void *pvParmeters );

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             ---------                                     */
/*                            | M a i n |                                    */
/*                             ---------                                     */
/*    This is the program entry point.                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int main( void )
{
	Val = 0;
	AT91C_BASE_AIC->AIC_EOICR = 0;
    // enable the clock of the PIO
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED2 ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, TP1|AMBE_RESET_|ECHOCANCEL ) ;
	AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, LED2|TP1|AMBE_RESET_|ECHOCANCEL ) ;

	TimerInit();
	UsartInit(UART_BAUD);
	
#ifndef TESTMODE
	Ambe2000Init();
	xTaskCreate( LedTask, ( const signed portCHAR * const ) "LED", 
					configMINIMAL_STACK_SIZE, NULL, LED_TASK_PRIORITY, ( xTaskHandle * ) NULL );
#else
	AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, LED1  );	//dont use PWM for LED1
	xTaskCreate( Led1TestTask, ( const signed portCHAR * const ) "LED1", 
					configMINIMAL_STACK_SIZE, NULL, LED_TASK_PRIORITY, ( xTaskHandle * ) NULL );
	xTaskCreate( Led2TestTask, ( const signed portCHAR * const ) "LED2", 
					configMINIMAL_STACK_SIZE, NULL, LED_TASK_PRIORITY+1, ( xTaskHandle * ) NULL );
#endif	
	vTaskStartScheduler();
	return 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             ---------------                               */
/*                            | L e d T a s k |                              */
/*                             ---------------                               */
/*    This tasks manages the overflow/underflow LED display.                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void LedTask( void *pvParmeters )
{
U32BIT led2cnt = 0;
 	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED2 );
	vTaskDelay(500);
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED2 );
    for( ;; )
    {
		vTaskDelay(10);
 		if(	GENERAL_STATUS_RUNNING == Status && CodecDataStarted && ChanDataStarted && QueueOverflow )
 		{
 			led2cnt++;
			if(led2cnt < 25)
			{	//red led on
				AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED2 );
			}
			else 
			{
				AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED2 );
				if(led2cnt > 30)
				{
	 				led2cnt = 0;
	 				QueueOverflow = FALSE;
				}
			}
		}
		else
		{
			led2cnt = 0;
			AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED2 );
		}
 	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             -----------------                             */
/*                            | L e d 1 T e s t |                            */
/*                             -----------------                             */
/*    This tasks manages the RED LED1 test sequencer.                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Led1TestTask( void *pvParmeters )
{
S32BIT timer;
U32BIT random;
    for( ;; )
    {
    	timer = 600;
    	random = (rand()>>3)&0x7F;
    	random += 20;
     	while(timer>0)
    	{
	 		vTaskDelay(random);
	 		timer -= random;
			TOGGLE_LED1();
    	}
		AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED1 );
// 		vTaskDelay(333);
 	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             -----------------                             */
/*                            | L e d 2 T e s t |                            */
/*                             -----------------                             */
/*    This tasks manages the GREEN LED2 test sequencer.                      */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Led2TestTask( void *pvParmeters )
{
S32BIT timer;
U32BIT random;
   for( ;; )
    {
    	timer = 500;
    	random = (rand()>>6)&0x3F;
    	random += 20;
     	while(timer>0)
    	{
	 		vTaskDelay(random);
	 		timer -= random;
			TOGGLE_LED2();
    	}
		AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED2 );
//  		vTaskDelay(500);

 	}
 
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                 -----------------------------------------                 */
/*                | v A p p l i c a t i o n I d l e H o o k |                */
/*                 -----------------------------------------                 */
/*    This is called during RTOS idle time.                                  */
/*    Maybe good place for watchdog petting.                                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void vApplicationIdleHook( void )
{
}
