/* ==========================================================================*/
/* - - - - - - - - - - - - -  T i m e r . c  - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-10   MW                                                */
/*.........................................................................  */
/* Digital Voice Dongle AT91SAM7256 Timer management                         */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "common.h"
#include "timer.h"
#include "irq.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | T i m e r I n i t |                                 */
/*                      -------------------                                  */
/*    Called to initialize the Timers.                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void TimerInit ( void )
{
int dummy;
    //* Open timer1	(GP timer)
	//* First, enable the clock of the TIMER
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_TC1 ) ;
	//* Disable the clock and the interrupts
	AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKDIS;
	AT91C_BASE_TC1->TC_IDR = 0xFFFFFFFF;
	//* Clear status bit
	dummy = AT91C_BASE_TC1->TC_SR;
	//* Set the Mode of the Timer Counter
	AT91C_BASE_TC1->TC_RC = 2400;
	AT91C_BASE_TC1->TC_CMR = AT91C_TC_CLKS_TIMER_DIV5_CLOCK|AT91C_TC_WAVESEL_UP_AUTO|
								AT91C_TC_WAVE; //48000 hz clk to timer1
	//* Enable the clock
	AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKEN;
    //* Config Timer 1 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_TC1, TIMER1_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, Timer1IrqService);
	AT91C_BASE_TC1->TC_IER = AT91C_TC_CPCS;  //  IRQ enable CPC
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_TC1);
	//* Start timer1
	AT91C_BASE_TC1->TC_CCR = AT91C_TC_SWTRG;
	
   //* Open timer2	(AMBE Message One Shot Timer)
	//* First, enable the clock of the TIMER
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_TC2 );
	//* Disable the clock and the interrupts
	AT91C_BASE_TC2->TC_CCR = AT91C_TC_CLKDIS;
	AT91C_BASE_TC2->TC_IDR = 0xFFFFFFFF;
	//* Clear status bit
	dummy = AT91C_BASE_TC2->TC_SR;
	//* Set the Mode of the Timer Counter
	AT91C_BASE_TC2->TC_RC = 0xFFFF;
	AT91C_BASE_TC2->TC_CMR = AT91C_TC_CLKS_TIMER_DIV5_CLOCK|AT91C_TC_CPCSTOP|AT91C_TC_WAVESEL_UP_AUTO|
								AT91C_TC_WAVE; //48000 hz clk to timer2
	//* Enable the clock
	AT91C_BASE_TC2->TC_CCR = AT91C_TC_CLKEN;
    //* Config Timer 2 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_TC2, TIMER2_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, Timer2IrqService);
	AT91C_BASE_TC2->TC_IER = AT91C_TC_CPCS;  //  IRQ enable CPC
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_TC2);
	
	//* Setup PWM channel 0 
	//* enable the clock of the PWM system
	//* Configure PIO controllers to periph mode for PWM
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 		((unsigned int) LED1  ) , // Peripheral A
 		0); // Peripheral B
	LedTimer = 0;
	LedTimerTog = 1;
 	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_PWMC ) ;

	*AT91C_PWMC_DIS = AT91C_PWMC_CHID0;
	*AT91C_PWMC_MR = 0;	//turn off DIVA and B
	*AT91C_PWMC_CH0_CMR  = 0xA; 	// (PWMC_CH0) MCLK/1024 update duty cycle
	*AT91C_PWMC_CH0_CPRDR = 450; 	// (PWMC_CH0) Channel Period Register 120Hz
	*AT91C_PWMC_CH0_CDTYR = 1; 		// (PWMC_CH0) Channel Duty Cycle Register
	*AT91C_PWMC_CH0_CUPDR  = 1; 	// (PWMC_CH0) Channel Update Register
	*AT91C_PWMC_ENA = AT91C_PWMC_CHID0;
	
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | S e t L e d P W M |                                 */
/*                      -------------------                                  */
/*    Called to Set the intensity of the LED. (range 1 to 449)               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SetLedPWM( U32BIT pwm)
{
	if(pwm < LED_MIN)
		pwm = LED_MIN;
	if(pwm > LED_MAX)
		pwm = LED_MAX;
	*AT91C_PWMC_CH0_CUPDR = pwm;
}
