/* ==========================================================================*/
/* - - - - - - - - - - - - - -  U s a r t . C   - - - - - - - - - - - - - - -*/
/* ==========================================================================*/
/*	Created   2007-07-01   M. Wheatley                                       */
/*	Modified  2007-07-01   MW                                                */
/*.........................................................................  */
/* DV Dongle AT91SAM7256 usart handler                                       */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "usart.h"
#include "ambe2000.h"
#include "irq.h"
#include <stdio.h>
#include <string.h>
 
/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/

#define USART_TASK_PRIORITY ( tskIDLE_PRIORITY + 3 )
#define QUEUE_SIZE 3

//* Standard Asynchronous Mode : 8 bits , 1 stop , no parity
#define ASYNC_MODE ( AT91C_US_USMODE_NORMAL + \
                        AT91C_US_NBSTOP_1_BIT + \
                        AT91C_US_PAR_NONE + \
                        AT91C_US_CHRL_8_BITS + \
                        AT91C_US_CLKS_CLOCK )


// define some offsets into msg buffer
#define MSG_SIZE 0			//location of msg size and type
#define MSG_TYPE 1
#define MSG_CITEM 2			//location of 16 bit CItem value
#define MSG_PARAM 4			//start of CItem parameters
#define MSG_DATAADR 2		//start of Data in Data msg

#define TXBUF_SIZE 4096

const char ID_DESC[] = "DV Dongle";

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
static BOOL SendResp;
static U32BIT TxBufEnd;
static U8BIT UsartTxBuf[TXBUF_SIZE];
U32BIT Status;
xSemaphoreHandle xUartSemaphore;
U8BIT UsartRxMsgA[MAXMSG_SIZE];
U8BIT UsartRxMsgB[MAXMSG_SIZE];
volatile U8BIT* pActiveMsg;

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
static void UsartTask( void *pvParmeters );
static void ParseHostMsg(U8BIT* pMsg);

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        --------------------                               */
/*                       |  U s a r t I n i t |                              */
/*                        --------------------                               */
/*    Called to initialize the USART 0 port.                                 */
/*  Rx and Tx done using DMA.                                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void UsartInit( U32BIT Rate )
{
	if( 0 == Rate)
	{
		*AT91C_US0_CR = AT91C_US_TXDIS | AT91C_US_RXDIS;
		*AT91C_US0_PTCR = AT91C_PDC_RXTDIS|AT91C_PDC_TXTDIS;
		*AT91C_US0_IDR = ~(0);
	}
	else
	{
		*AT91C_US0_IDR = ~(0);
		vSemaphoreCreateBinary( xUartSemaphore );
 		//* Configure PIO controllers to periph mode
 		AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 			((unsigned int) RXD    ) |
 			((unsigned int) TXD    ),// Peripheral A
 			0); // Peripheral B
		pActiveMsg = UsartRxMsgA;
	 	*AT91C_US0_RPR = (U32BIT)pActiveMsg;
		*AT91C_US0_RCR = 2;
		*AT91C_US0_RNPR = 0;
		*AT91C_US0_RNCR = 0;
		RxMsgLength = -1;	//flag msg length as not yet received
		TxBufEnd = 0;
		
		*AT91C_US0_TPR = 0;
		*AT91C_US0_TCR = 0;
		*AT91C_US0_TNPR = 0;
		*AT91C_US0_TNCR = 0;
		*AT91C_US0_PTCR = AT91C_PDC_RXTEN|AT91C_PDC_TXTEN;

   		AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_US0 ) ;
		// Usart Configure
 		AT91C_BASE_US0->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX;
 		AT91C_BASE_US0->US_CR = AT91C_US_RSTSTA;
  		AT91C_BASE_US0->US_BRGR = MCK/(16*Rate);
		AT91C_BASE_US0->US_MR = ASYNC_MODE;
		AT91C_BASE_US0->US_TTGR = 0;	//no inter tx spacing
		AT91C_BASE_US0->US_RTOR = 0xFFFF;
		// Enable usart
		AT91C_BASE_US0->US_CR = AT91C_US_RXEN|AT91C_US_STTTO|AT91C_US_TXEN;
		
		xTaskCreate( UsartTask, ( const signed portCHAR * const ) "USART", 
					configMINIMAL_STACK_SIZE, NULL, USART_TASK_PRIORITY, ( xTaskHandle * ) NULL );

	    	//* Enable USART error and Rx/Tx DMA interrupts
		AT91F_US_EnableIt(AT91C_BASE_US0, AT91C_US_ENDRX|AT91C_US_TIMEOUT);
	    //* Config Usart interrupt
		AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_US0, USART_INTERRUPT_LEVEL,
								AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, UsartIrqService);
		AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_US0);
	}
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | U s a r t T a s k |                                 */
/*                      -------------------                                  */
/*    Task that process usart rx bytes from ping pong buffers                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void UsartTask( void *pvParmeters )
{
	Status = GENERAL_STATUS_STOPPED;
 	xSemaphoreTake(xUartSemaphore, ( portTickType )0);
    for( ;; )	//loop forever in this task
    {
		if( xSemaphoreTake( xUartSemaphore, ( portTickType )5000 ) == pdTRUE )
		{	//Msg ready from usart
			if(pActiveMsg == UsartRxMsgA)
				ParseHostMsg(UsartRxMsgB);
			else
				ParseHostMsg(UsartRxMsgA);
		}
		else	//nothing coming in from host so turn off
		{
			Status = GENERAL_STATUS_STOPPED;
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------------                            */
/*                     | S e n d U s a r t M s g |                           */
/*                      -------------------------                            */
/*    Function that sends a Msg buffer of length Len to the usart            */
/* If a DMA is still in progress, the msg is appended to the buffer end      */
/* Dont call from IRQ or ka-boom.                                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SendUsartMsg(U8BIT* Buf, U32BIT Len)
{
U32BIT i;
U32BIT bytesleft;
	vTaskSuspendAll();
	*AT91C_US0_PTCR = AT91C_PDC_TXTDIS;		//freeze DMA while updating
	while( (AT91C_BASE_US0->US_CSR & AT91C_US_TXEMPTY) == 0 ){};
	bytesleft = *AT91C_US0_TCR;
	if( 0 == bytesleft )	//Tx DMA is not active so start new DMA
	{
		*AT91C_US0_TPR = (U32BIT)UsartTxBuf;
		for(i=0; i<Len; i++)
			UsartTxBuf[i] = Buf[i];
		TxBufEnd = Len;
	}
	else	//if still active then just stick new msg at end of buffer
	{
		for(i=0; i<Len; i++)
			UsartTxBuf[i+TxBufEnd] = Buf[i];
		TxBufEnd += Len;
	}
	*AT91C_US0_TCR = Len + bytesleft;
	if( *AT91C_US0_TCR > (TXBUF_SIZE-1) )
	{	//if too many bytes just reset buffer and dont dma
		*AT91C_US0_TCR = 0;
		TxBufEnd = 0;
	}
	*AT91C_US0_PTCR = AT91C_PDC_TXTEN;
	xTaskResumeAll();
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    -------------------------                              */
/*                   | P a r s e H o s t M s g |                             */
/*                    -------------------------                              */
/*    This routine is called to parse a new message from the Host.           */
/* INPUTS: The Msg Buffer 'pMsg' containing the new msg from the Host        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ParseHostMsg(U8BIT* pMsg)
{
U32BIT i,j;
U32BIT mType;
U32BIT mCitem;
U32BIT mLength;
t32BITS tmp;
U8BIT ch;
U8BIT* pBuf;

	mType = (U32BIT)(pMsg[MSG_TYPE] & 0xE0);	//strip out just type field
	mCitem = (U32BIT)pMsg[MSG_CITEM+1];		//get CItem
	mCitem <<= 8;
	mCitem |= (U32BIT)pMsg[MSG_CITEM];
	SendResp = TRUE;
	if( mType == TYPE_HOST_SET_CITEM )
	{	// Is a message from host to set a control item value
		pMsg[MSG_TYPE] = TYPE_TARG_RESP_CITEM;
		switch( mCitem )
		{
			case CI_RUN_STATE:
				if(RUN_STATE_START == pMsg[MSG_PARAM])
					Status = GENERAL_STATUS_RUNNING;
				else
					Status = GENERAL_STATUS_STOPPED;
				CodecDataStarted = FALSE;
				ChanDataStarted = FALSE;
				InitInFrameQueue();
 				QueueOverflow = FALSE;
				break;
			case CI_UPDATE_MODE_CONTROL:
				if( (pMsg[MSG_PARAM+2] == BOOTSECURE0 ) &&
					(pMsg[MSG_PARAM+3] == BOOTSECURE1 ) &&
					(pMsg[MSG_PARAM+4] == BOOTSECURE2 ) &&
					(pMsg[MSG_PARAM+5] == BOOTSECURE3 ) )
				{	//is valid boot command
					if( pMsg[MSG_PARAM+1] == CI_UPDATE_MODE_CONTROL_ENTER )
					{	//enter boot mode via a software induced reset
						SendUsartMsg(pMsg, (U32BIT)pMsg[MSG_SIZE] );	//ack msg first
						vTaskSuspendAll();
						while( (AT91C_BASE_US0->US_CSR & AT91C_US_TXEMPTY) == 0 )
							vTaskDelay( 100 );	//wait for ack to be sent
						*AT91C_RSTC_RCR = ((unsigned int) 0xA5 << 24)|AT91C_RSTC_PROCRST|AT91C_RSTC_PERRST;	// SW reset back to boot code
					}
					else
					{
						pMsg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK 
						pMsg[MSG_TYPE] = 0; 
					}
				}
				break;
			default:
				pMsg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK 
				pMsg[MSG_TYPE] = 0; 
				break;
		}
	}
	else if( mType == TYPE_HOST_REQ_CITEM )
	{	// Is a message from host requesting a control item value
		pMsg[MSG_TYPE] = TYPE_TARG_RESP_CITEM;
		switch(mCitem)
		{	
			case CI_GENERAL_INTERFACE_NAME:
				i = 0;
				do{
					ch = ID_DESC[i];
					pMsg[MSG_PARAM+i] = ch;
					i++;
				}while( (ch != 0) &&  (i<20) );
				pMsg[MSG_SIZE] = 4+i;
				break;
			case CI_GENERAL_INTERFACE_SERIALNUM:
				i = 0;
				do{
					ch = SERIAL_NUMBER[i];
					pMsg[MSG_PARAM+i] = ch;
					i++;
				}while( (ch != 0) &&  (i<9) );
				pMsg[MSG_SIZE] = 4+i;
				break;
			case CI_GENERAL_INTERFACE_VERSION:
				pMsg[MSG_SIZE] = CI_GENERAL_INTERFACE_VERSION_SETRESPLEN;
				pMsg[MSG_PARAM] = (DVDONGLE_INTERFACE_VERSION&0x00FF);
				pMsg[MSG_PARAM+1] = ((DVDONGLE_INTERFACE_VERSION>>8)&0x00FF);
				break;
			case CI_GENERAL_HARDFIRM_VERSION:
				pMsg[MSG_SIZE] = CI_GENERAL_HARDFIRM_VERSION_SETRESPLEN;
				if(pMsg[MSG_PARAM+0] == 0)	//boot code version
				{
					pMsg[MSG_PARAM+1] = *BOOT_VERSION & 0x00FF;
					pMsg[MSG_PARAM+2] = (*BOOT_VERSION>>8) & 0x00FF;
				}
				else if(pMsg[MSG_PARAM+0] == 1) //flash version
				{	
					pMsg[MSG_PARAM+1] =  FIRMWARE_VERSION & 0x00FF;
					pMsg[MSG_PARAM+2] =  (FIRMWARE_VERSION>>8) & 0x00FF;
				}
				else if(pMsg[MSG_PARAM+0] == 2) 	//HW version
				{
					pMsg[MSG_PARAM+1] = *HW_VERSION&0x00FF;
					pMsg[MSG_PARAM+2] = (*HW_VERSION>>8)&0x00FF;
				}
				break;
			case CI_GENERAL_STATUS_CODE:
				pMsg[MSG_SIZE] = CI_GENERAL_STATUS_CODE_SETRESPLEN;
				pMsg[MSG_PARAM] = Status;
				break;
			case CI_UPDATE_MODE_PARAMS:
				pMsg[MSG_SIZE] = CI_UPDATE_MODE_PARAMS_SETRESPLEN;
				tmp.all = FLASH_SIZE;
				pMsg[MSG_PARAM+1] = tmp.bytes.b0;
				pMsg[MSG_PARAM+2] = tmp.bytes.b1;
				pMsg[MSG_PARAM+3] = tmp.bytes.b2;
				pMsg[MSG_PARAM+4] = tmp.bytes.b3;
				tmp.all = FLASH_PAGE_SIZE;
				pMsg[MSG_PARAM+5] = tmp.bytes.b0;
				pMsg[MSG_PARAM+6] = tmp.bytes.b1;
				pMsg[MSG_PARAM+7] = tmp.bytes.b2;
				pMsg[MSG_PARAM+8] = tmp.bytes.b3;
				tmp.all = FLASH_SECTOR_SIZE;
				pMsg[MSG_PARAM+9] = tmp.bytes.b0;
				pMsg[MSG_PARAM+10] = tmp.bytes.b1;
				pMsg[MSG_PARAM+11] = tmp.bytes.b2;
				pMsg[MSG_PARAM+12] = tmp.bytes.b3;
				break;
			default:
				pMsg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK 
				pMsg[MSG_TYPE] = 0; 
				break;
		}
	}
	else if( mType == TYPE_HOST_DATA_ITEM0 )
	{	//here with new Codec data packet from host
		if(!CodecDataStarted)	//if is first time since started so resync fifo
		{
			CodecTxTail = CodecTxHead+1;
			CodecTxTail &= (CODEC_QUE_SIZE-1);
			QueueOverflow = FALSE;
			CodecDataStarted = TRUE;
		}
		mLength = (U32BIT)pMsg[MSG_SIZE+1];		//get length
		mLength <<= 8;
		mLength |= (U32BIT)pMsg[MSG_SIZE];
		mLength &= LENGTH_MASK;
		if(  mLength == CODEC_BUF_SIZE*2 )
		{
			//place in codec queue
			pBuf = (U8BIT*)CodecTxQue[CodecTxHead].Data;
			for(i=MSG_DATAADR,j=0; i<mLength; i++,j++)
				pBuf[j] = pMsg[i];
			CodecTxHead++;
			CodecTxHead &= (CODEC_QUE_SIZE-1);
			if( CodecTxTail==CodecTxHead)	//queue over run
			{	//move pointer back 
				CodecTxHead -= (CODEC_QUE_SIZE/4);
				CodecTxHead &= (CODEC_QUE_SIZE-1);
				QueueOverflow = TRUE;
			}
		}
		SendResp = FALSE;
	}
	else if( mType == TYPE_HOST_DATA_ITEM1 )
	{	//here with new AMBE frame data packet from host
		if(!ChanDataStarted)	//if is first time since started so resync fifo
		{
			ChanTxHead = ChanTxTail+1;
			ChanTxHead &= (CHANNEL_QUE_SIZE-1);
			QueueOverflow = FALSE;
			ChanDataStarted = TRUE;
		}
		mLength = (U32BIT)pMsg[MSG_SIZE+1];		//get length
		mLength <<= 8;
		mLength |= (U32BIT)pMsg[MSG_SIZE];
		mLength &= LENGTH_MASK;
		if( mLength == sizeof(tInFrame) + 2)
		{
			for(i=MSG_DATAADR+24; i<MSG_DATAADR+48; i+=2)
			{	//change byte data fields to big endian
				ch = pMsg[i];
				pMsg[i] = pMsg[i+1];
				pMsg[i+1] = ch;
			}
			pBuf = (U8BIT*)InFrame[ChanTxHead].All;
			for(i=MSG_DATAADR,j=0; i<mLength; i++,j++)
				pBuf[j] = pMsg[i];
			ChanTxHead++;
			ChanTxHead &= (CHANNEL_QUE_SIZE-1);
			if( ChanTxHead==ChanTxTail)	//queue over run
			{	//move pointer back half way
				ChanTxHead -= (CHANNEL_QUE_SIZE/4);
				ChanTxHead &= (CHANNEL_QUE_SIZE-1);
				QueueOverflow = TRUE;
			}
		}
		SendResp = FALSE;
	}
	else
	{
		SendResp = FALSE;
	}
	if(	SendResp )
	{
		SendUsartMsg(pMsg, (U32BIT)pMsg[MSG_SIZE] );
		SendResp = FALSE;
	}
}



