/*---------------------------------------------------------------------------*/
/*-                       Linker Script File                                 */
/*---------------------------------------------------------------------------*/
/* Defines Memory Map for AT91SAM7256 application code.     	     	     */
/*---------------------------------------------------------------------------*/
/*- File source          : ***.ld                                            */
/*- Object               : Linker Script File for User Flash Code            */
/*- Compilation flag     : Runs in ARM Mode                                  */
/*-                                                                          */
/*- 1.0 2006/01/29 MSW   : Creation S256                                     */
/*---------------------------------------------------------------------------*/


/* ==========================================================================*/
/*                                                                           */
/*                         MEMORY MAP                                        */
/*      .--------> |---------------------------------|                       */
/*      .          |                                 |0x0020FFFF             */
/*      .          |                                 |                       */
/*      .          |              RAM                |                       */
/*      .          |                                 |                       */
/*     RAM         |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .--------->|---------------------------------|0x00200000             */
/*                                                                           */
/*                                                                           */
/*      .--------> |---------------------------------|                       */
/*      .          |                                 |0x0013FFFF             */
/*      .          |                                 |                       */
/*      .          |    User  Flash Blocks           |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*    Flash        |                                 |                       */
/*      .          |.................................|0x00104040             */
/*      .          |   User Vectors and tables       |0x0010403F             */
/*      .          |   to be copied to RAM before    |                       */
/*      .          |  executing user code            |                       */
/*      .          |.................................|0x00104000             */
/*      .          |                                 |0x00103FFF             */
/*      .          | Protected BootLoader Block      |                       */
/*      .          |                                 |                       */
/*      .--------->|---------------------------------|0x00100000             */
/*                                                                           */
/*                                                                           */
/*      .--------> |---------------------------------|                       */
/*      .          |                                 |0x0003FFFF (0x0000FFFF)*/
/*      .          |                                 |           after remap)*/
/*      .          |    Flash or Ram after remap     |                       */
/*      .          |                                 |                       */
/*    Flash        |                                 |                       */
/*     or          |                                 |                       */
/*  (Ram after     |                                 |                       */
/*    remap)       |                                 |                       */
/*      .          |                                 |                       */
/*      .          |.................................|0x00000040             */
/*      .          | Vectors and jump tables         |0x0000003F             */
/*      .          | Copied from user flash before   |                       */
/*      .          |  jumping to user code           |                       */
/*      .--------->|---------------------------------|0x00000000             */
/* ==========================================================================*/


/*
// <h> Memory Configuration
//   <h> Code (Read Only)
//     <o>  Start <0x0-0xFFFFFFFF>
//     <o1> Size  <0x0-0xFFFFFFFF>
//   </h>
//   <h> Data (Read/Write)
//     <o2> Start <0x0-0xFFFFFFFF>
//     <o3> Size  <0x0-0xFFFFFFFF>
//   </h>
//   <h> Top of Stack (Read/Write)
//     <o4> STACK <0x0-0xFFFFFFFF>
//   </h>
// </h>
*/

/* Memory Definitions (user application is called after memory REMAP )*/

MEMORY
{
  BOOT (rx) : ORIGIN = 0x00100000, LENGTH = 0x00004000	/* use first 16K AT91SAM7256 flash block */
  CODE (rx) : ORIGIN = 0x00104000, LENGTH = 0x0000F000  /* AT91SAM7256 has 16K flash blocks */
  DATA (rw) : ORIGIN = 0x00200040, LENGTH = 0x00010000	/*first 64 bytes get copied by bootloader */
  STACK (rw) : ORIGIN = 0x00210000,LENGTH = 0x00000000
}


/* Section Definitions */

SECTIONS
{
 /* first section is .text which is used for code */
  . = 0x00104000;
  _text = .;
  .text : 
  {
    *(.startup)                /* CStartup must be located at 0x00104000 */
    *(.text)                   /* remaining code */
    *(.glue_7t)
    *(.glue_7)

  } >CODE =0

  . = ALIGN(4);

  /* .rodata section which is used for read-only data (constants) */

  .rodata :
  {
    *(.rodata)
    *(.rodata*)
  } >CODE

  . = ALIGN(4);

  _etext = . ;
  PROVIDE (etext = .);

  /* .data section which is used for initialized data */

  .data : AT (_etext)
  {
    _data = . ;
    *(.data)
    SORT(CONSTRUCTORS)
  } >DATA
  . = ALIGN(4);

  _edata = . ;
   PROVIDE (edata = .);

  /* .bss section which is used for uninitialized data */

  .bss :
  {
    __bss_start = . ;
    __bss_start__ = . ;
    *(.bss)
  } >DATA
  . = ALIGN(4);
  __bss_end__ = . ;
  __bss_end__ = . ;
  _end = .;
    . = ALIGN(4);
   .int_data :  
   { 
   		*(.internal_ram_top) 
   }> STACK 

  PROVIDE (end = .);
  
}
