/* ==========================================================================*/
/* - - - - - - - - - - -  A m b e 2 0 2 0 . H   - - - - - - - - - - - - - -  */
/* ==========================================================================*/
/*    Definition File for the Ambe2020.c module                              */
/*  Created   2007-07-01	msw                                              */
/*  Modified  2007-07-01    msw                                              */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
#ifndef AMBE2020_H_
#define AMBE2020_H_
#include "common.h"

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/*---------------------------------------------------------------------------*/
/*------------------> G L O B A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define XFERSTATE_FINISHED	0
#define XFERSTATE_LOOKING_FOR_SYNC	1
#define XFERSTATE_RECEIVING	2

#define AMBE2020_FRAME_SYNC	0x13EC
#define AMBE2020_IDLE_WORD	0x0000

#define TIMER2_DATARDY 1	//~41uS
#define TIMER2_SYNCSEARCH 48	//1mS
#define TIMER2_DELAY_TO_SYNC_SEARCH (768)	//16mSec

#define CODEC_DATA_LENGTH 160	//number of 16 bit data words per block
#define CODEC_BUF_HDR 1			//number of 16 bit header words that go in front of data
#define CODEC_BUF_SIZE (CODEC_BUF_HDR+CODEC_DATA_LENGTH)
#define CODEC_QUE_SIZE 16	//must be pwr of 2 for "AND" wrap around

#define CHANNEL_QUE_SIZE 16	//must be pwr of 2 for "AND" wrap around
/*---------------------------------------------------------------------------*/
/*-------------------------> T y p e d e f s <-------------------------------*/
/*---------------------------------------------------------------------------*/

typedef union
{
	struct
	{
		U16BIT Hdr;
		U16BIT CW1;
		U16BIT Rate0;
		U16BIT Rate1;
		U16BIT Rate2;
		U16BIT Rate3;
		U16BIT Rate4;
		U16BIT Unused1;
		U16BIT Unused2;
		U16BIT Unused3;
		U16BIT DTMFCtrl;
		U16BIT CW2;
		U16BIT ChData[12];
	}fld;
	U16BIT All[24];
}tInFrame;

typedef union
{
	struct
	{
		U16BIT Hdr;
		U16BIT CW1;
		U16BIT Rate0;
		U16BIT Rate1;
		U16BIT Rate2;
		U16BIT Rate3;
		U16BIT Rate4;
		U16BIT Unused1;
		U16BIT Unused2;
		U16BIT BitErrors;
		U16BIT DTMFCtrl;
		U16BIT CW2;
		U16BIT ChData[12];
	}fld;
	U16BIT All[24];
}tOutFrame;

typedef	struct
{
	U16BIT Hdr;
	U16BIT Data[CODEC_DATA_LENGTH];
}tCodecData;


/*---------------------------------------------------------------------------*/
/*------------------> E x p o s e d    F u n c t i o n s <-------------------*/
/*---------------------------------------------------------------------------*/
void Ambe2000Init( );
void InitInFrameQueue(void);

/*---------------------------------------------------------------------------*/
/*------------------> E x p o s e d    V a r i a b l e s <-------------------*/
/*---------------------------------------------------------------------------*/
extern BOOL QueueOverflow;
extern BOOL CodecDataStarted;
extern BOOL ChanDataStarted;
extern volatile U32BIT XferState;
extern volatile tCodecData* pActiveCodecBuf;
extern U32BIT SSCTransferIndex;
extern volatile U32BIT ChanTxHead;
extern volatile U32BIT ChanTxTail;
extern tInFrame InFrame[CHANNEL_QUE_SIZE];
extern tOutFrame OutFrame;
extern xSemaphoreHandle xAmbeCodecDataRdySemaphore;
extern xSemaphoreHandle xAmbeChanDataRdySemaphore;
extern volatile U32BIT CodecTxHead;
extern volatile U32BIT CodecTxTail;
extern tCodecData CodecRxBufA;
extern tCodecData CodecRxBufB;
extern tCodecData CodecTxQue[CODEC_QUE_SIZE];

#endif /*AMBE2020_H_*/
