/* ==========================================================================*/
/* - - - - - - - - - - - - -   B o a r d . h   - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    Definition File for the DV Dongle board                                */
/*  Created   2007-03-09   msw                                               */
/*	Modified  2007-09-09   MW                                                */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
#ifndef Board_h
#define Board_h
#include "common.h"

#define FIRMWARE_VERSION 112		//firmware revision * 100 ( 123 -> 1.23 )



#include "AT91SAM7S256.h"
#define __inline extern inline
#include "lib_AT91SAM7S256.h"
#define __ramfunc

#define AT91C_US_ASYNC_MODE ( AT91C_US_USMODE_NORMAL + AT91C_US_NBSTOP_1_BIT + AT91C_US_PAR_NONE + AT91C_US_CHRL_8_BITS + AT91C_US_CLKS_CLOCK )

/*---------------------------*/
/*  Memories Definition      */
/*---------------------------*/
// The AT91SAM7S256 has a 64-Kbyte SRAM bank, and 256K-Byte Flash

#define  INT_SARM           0x00200000
#define  INT_SARM_REMAP	    0x00000000

#define  INT_FLASH          0x00000000
#define  INT_FLASH_REMAP    0x00100000

#define  BOOTFLASH_START    0x00000000
#define  BOOTFLASH_END    	0x00003FFF

#define  FLASH_PAGE_NB	    1024
#define  FLASH_PAGE_LOCK    64
#define  FLASH_PAGE_SIZE    256

#define FLASH_SECTOR_SIZE (FLASH_PAGE_LOCK*FLASH_PAGE_SIZE)
#define FLASH_SIZE (FLASH_PAGE_NB*FLASH_PAGE_SIZE)


#define BOOT_VERSION  ( (U16BIT*)0x00100040) // Location of boot code version number in boot flash memory
#define HW_VERSION  ( (U16BIT*)0x00100042) // Location of HW version number in boot flash memory
#define SERIAL_NUMBER  ( (U8BIT*)0x00100044) // Location of Serial number string in boot flash memory


/*---------------------*/
/* I/O Pin Definitions */
/*---------------------*/
/*----------------------------------------------------------------------------------*/
/*  PORT A I/O Pin Definitions                    AT91SAM7 PIO Multiplexing         */
/*                                      PIO     Flash    Periph A   Periph B   PIN  */
/*----------------------------------------------------------------------------------*/
#define LED1            (1<<0)  /*       PA0 / PGMEN0     PWM0      TIOA0     48    */
#define LED2            (1<<1)  /*       PA1 / PGMEN1     PWM1      TIOB0     47    */
#define PA2             (1<<2)  /*       PA2              PWM2      SCK0      44    */
#define PA3             (1<<3)  /*       PA3              TWD       NPCS3     43    */
#define ENCEN           (1<<4)  /*       PA4              TWCK      TCLK0     36    */
#define RXD             (1<<5)  /*       PA5              RXD0      NPCS3     35    */
#define TXD             (1<<6)  /*       PA6              TXD0      PCK0      34    */
#define RTS0            (1<<7)  /*       PA7              RTS0      PWM3      32    */
#define CTS0            (1<<8)  /*       PA8              CTS0      ADTRG     31    */
#define PA9             (1<<9)	/*       PA9              DRXD      NPCS1     30    */
#define PA10            (1<<10) /*       PA10             DTXD      NPCS2     29    */
#define CHAN_STB        (1<<11) /*       PA11             NPCS0     PWM0      28    */
#define CHAN_TXDATA     (1<<12) /*       PA12             MISO      PWM1      27    */
#define CHAN_RXDATA     (1<<13) /*       PA13             MOSI      PWM2      22    */
#define CHAN_CLK        (1<<14) /*       PA14             SPCK      PWM3      21    */
#define SSC_TF          (1<<15) /*       PA15             TF        TIOA1     20    */
#define SSC_TK          (1<<16) /*       PA16             TK        TIOB1     19    */
#define SSC_TD          (1<<17) /*       PA17             TD        PCK1       9    */
#define SSC_RD          (1<<18) /*       PA18             RD        PCK2      10    */
#define SLIP_EN         (1<<19) /*       PA19             RK        FIQ       13    */
#define ECHOCANCEL      (1<<20) /*       PA20             RF        IRQ0      16    */
#define PA21            (1<<21) /*       PA21             RXD1      PCK1      11    */
#define AMBE_RESET_     (1<<22) /*       PA22             TXD1      NPCS3     14    */
#define EPR_            (1<<24) /*       PA24             RTS1      PWM1      23    */
#define PA25            (1<<25) /*       PA25             CTS1      PWM2      25    */
#define PA26            (1<<26) /*       PA26             DCD1      TIOA2     26    */
#define TP1             (1<<27) /*       PA27             DTR1      TIOB2     37    */
#define PA28            (1<<28) /*       PA28             DSR1      TCLK1     38    */
#define PA29            (1<<29) /*       PA29             RI1       TCLK2     41    */
#define PA30            (1<<30) /*       PA30             IRQ1      NPCS2     42    */
#define PA31            (1<<31) /*       PA31             NPCS1     PCK2      52    */

/*------------------*/
/* USART Definition */
/*------------------*/
#define UART_BAUD	   230400   // Baud rate

/*--------------*/
/* Master Clock */
/*--------------*/
#define EXT_OC          16384000   // External oscillator MAINCK
#define MCK             49152000   // MCK = [ (EXT_OC/5) * 30 ] / 2 ( see CStartup_SAM7.c)
#define MCKKHz          (MCK/1000) //

#define TOGGLE_LED1()	{													\
						if( AT91F_PIO_GetInput(AT91C_BASE_PIOA) & LED1 )	\
							AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED1 ); \
						else												\
							AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED1 );	\
						}
#define TOGGLE_LED2()	{													\
						if( AT91F_PIO_GetInput(AT91C_BASE_PIOA) & LED2 )	\
							AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED2 ); \
						else												\
							AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED2 );	\
						}



#endif /* Board_h */
