#if !defined( __ASCPDEFS_H__ )
#define __ASCPDEFS_H__
/* ==========================================================================*/
/*  - - - - - - -   P R O T O C O L D E F S . H   - - - - - - - - - - - - -  */
/* ==========================================================================*/
/*    Definition File for the DV Dongle protocol and Control Items           */
/*  Created 2007-07-07														 */
/*  Modified 2007-09-22		msw    added                                     */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
#define DVDONGLE_INTERFACE_VERSION 2

/*---------------------------------------------------------------------------*/
/*---------------------> Message Header Defines <----------------------------*/
/*---------------------------------------------------------------------------*/
#define LENGTH_MASK 0x1FFF			/* mask for message length               */
#define TYPE_MASK 0xE0				/* mask for msg type bits of header      */

#define TYPE_HOST_SET_CITEM (0<<5)
#define TYPE_HOST_REQ_CITEM (1<<5)
#define TYPE_HOST_REQ_CITEM_RANGE (2<<5)
#define TYPE_HOST_DATA_ITEM0 (4<<5)		//codec data item
#define TYPE_HOST_DATA_ITEM1 (5<<5)		//AMBE Frame data item
#define TYPE_HOST_DATA_ITEM2 (6<<5)
#define TYPE_HOST_DATA_ITEM3 (7<<5)

#define TYPE_TARG_RESP_CITEM (0<<5)
#define TYPE_TARG_UNSOLICITED_CITEM (1<<5)
#define TYPE_TARG_RESP_CITEM_RANGE (2<<5)

#define TYPE_TARG_DATA_ITEM0 (4<<5)		//codec data item
#define TYPE_TARG_DATA_ITEM1 (5<<5)		//AMBE Frame data item
#define TYPE_TARG_DATA_ITEM2 (6<<5)
#define TYPE_TARG_DATA_ITEM3 (7<<5)

#define TYPE_DATA_ITEM_ACK (3<<5)
 #define DATA_ITEM_ACK_LENGTH (3)

/*  2 byte NAK response to any unimplemented messages */
#define TARG_RESP_NAK (0x0002)

#define MAX_MSG_LENGTH (8192+2)
/*---------------------------------------------------------------------------*/
/*----------------------> Control Item Defines <-----------------------------*/
/*---------------------------------------------------------------------------*/

/*  General Interface Control Items  */
#define CI_GENERAL_INTERFACE_NAME 0x0001
 #define CI_GENERAL_INTERFACE_NAME_REQLEN 4

#define CI_GENERAL_INTERFACE_SERIALNUM 0x0002
 #define CI_GENERAL_INTERFACE_SERIALNUM_REQLEN 4

#define CI_GENERAL_INTERFACE_VERSION 0x0003
 #define CI_GENERAL_INTERFACE_VERSION_REQLEN 4
 #define CI_GENERAL_INTERFACE_VERSION_SETRESPLEN 6

#define CI_GENERAL_HARDFIRM_VERSION 0x0004
 #define CI_GENERAL_HARDFIRM_VERSION_REQLEN 5
 #define CI_GENERAL_HARDFIRM_VERSION_SETRESPLEN 7

#define CI_GENERAL_STATUS_CODE 0x0005
 #define CI_GENERAL_STATUS_CODE_REQLEN 4
 #define CI_GENERAL_STATUS_CODE_SETRESPLEN 5
  #define GENERAL_STATUS_STOPPED 0x00
  #define GENERAL_STATUS_RUNNING 0x01
  #define GENERAL_STATUS_BOOTIDLE 0x0E
  #define GENERAL_STATUS_BOOTBUSY 0x0F
  #define GENERAL_STATUS_BOOTERROR 0x80

/*  DV Dongle Specific Control Items  */
#define CI_RUN_STATE 0x0018
 #define CI_RUN_STATE_REQLEN 5
 #define CI_RUN_STATE_SETRESPLEN 5
   #define RUN_STATE_STOP 0x00
   #define RUN_STATE_START 0x01

/*  DV Dongle Specific Control Items  */
#define CI_DVDONGLE_SETUP 0x0034
 #define CI_DVDONGLE_SETUP_REQLEN 7
 #define CI_DVDONGLE_SETUP_SETRESPLEN 7
 	//First parameter byte is compressed data format
   #define DVDONGLE_SETUP_AMBEDATAMODE 0x00
   #define DVDONGLE_SETUP_DSTARDATAMODE 0x01
 	//Second parameter byte is compressed data FIFO size( 2 to 32)
 	//Third parameter byte is uncompressed codec data FIFO size(2 to 32)
 

/*  Software/Firmware Update MODE Control Items  */
#define CI_UPDATE_MODE_CONTROL 0x0300
 #define CI_UPDATE_MODE_CONTROL_SETREQLEN 10
 #define CI_UPDATE_MODE_CONTROL_SETRESPLEN 10
 #define CI_UPDATE_MODE_CONTROL_ENTER 0
 #define CI_UPDATE_MODE_CONTROL_START 1
 #define CI_UPDATE_MODE_CONTROL_END 2
 #define CI_UPDATE_MODE_CONTROL_ABORT 3
 #define CI_UPDATE_MODE_CONTROL_ERASE 4


 #define BOOTSECURE0 0x5A	// The next 4 bytes are security codes for entering
 #define BOOTSECURE1 0xFF	// the programming mode
 #define BOOTSECURE2 0xA5
 #define BOOTSECURE3 0x00

#define CI_UPDATE_MODE_PARAMS 0x0302
 #define CI_UPDATE_MODE_PARAMS_SETREQLEN 5
 #define CI_UPDATE_MODE_PARAMS_SETRESPLEN 17

#endif //!defined( __ASCPDEFS_H__ )
